// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: ModeCreateFigPoly.java
// Classes: ModeCreateFigPoly
// Original Author: jrobbins@ics.uci.edu
// $Id: ModeCreateFigPoly.java,v 1.6 1997/01/24 19:15:31 jrobbins Exp $

package uci.graphedit;

import java.awt.Event;

/** A Mode to interpert user input while creating a FigPoly. All of
 * the actual event handling is inherited from ModeCreate. This class
 * just implements the differences needed to make it specific to
 * Polys. */

public class ModeCreateFigPoly extends ModeCreate {

  public static final int GRIP_SIZE = 4;

  protected int _npoints = 0;
  protected int _lastX, _lastY, _startX, _startY;

  public String instructions() {
    return "Click to add a point; Double-click to finish";
  }

  
  /** Create a new FigRect instance based on the given mouse down
   * event and the state of the parent Editor. */
  public DiagramElement createNewItem(Event e) {
    FigPoly p = new FigPoly(e.x, e.y, parent.graphAttrs());
    p.addPoint(e.x, e.y); // add the first point twice
    _startX = _lastX = e.x; _startY = _lastY = e.y;
    _npoints = 2;
    return p;
  }

  ////////////////////////////////////////////////////////////////
  // Event handling methods

  public boolean mouseDown(Event e, int x, int y) {
    if (_npoints == 0) return super.mouseDown(e, x, y);
    if (!nearLast(x, y)) {
        parent.damaged(newItem);
        ((FigPoly)newItem).addPoint(x, y);
        _npoints++;
        parent.damaged(newItem);
    }
    return true;
  }

  public boolean mouseUp(Event e, int x, int y) {
    if (_npoints > 2 && nearLast(x, y)) {
        FigPoly p = (FigPoly) newItem;
        parent.damaged(newItem);
        p.moveVertex(p.npoints - 1, _startX, _startY);
        _npoints = 0;
        parent.damaged(p);
        parent.add(p);
        parent.selectItem(p);
        newItem = null;
        done();
        return true;
    }
    _lastX = x; _lastY = y;
    return true;
  }

  public boolean mouseMove(Event e, int x, int y) {
    return mouseDrag(e, x, y);
  }

  public boolean mouseDrag(Event e, int x, int y) {
    if (_npoints == 0) return false;
    FigPoly p = (FigPoly)newItem;
    parent.damaged(newItem); // startTrans?
    p.moveVertex(p.npoints - 1, x, y);
    parent.damaged(newItem); // endTrans?
    return true;
  }

protected boolean nearLast(int x, int y) {
    return x > _lastX - GRIP_SIZE &&
           x < _lastX + GRIP_SIZE &&
           y > _lastY - GRIP_SIZE &&
           y < _lastY + GRIP_SIZE;
}

} /* end class ModeCreateFigPoly */

